from __future__ import absolute_import
from __future__ import unicode_literals

import argparse

from pip_custom_platform.install import install
from pip_custom_platform.wheel import wheel


def add_shared_arguments(parser):
    parser.add_argument(
        '--platform', help='Custom platform name (required).', required=True,
    )


def main(argv=None):
    parser = argparse.ArgumentParser(
        description=(
            'pip+wheel wrapper which allows you to choose a custom platform '
            'name for building, downloading, and installing wheels.\n\n'
            'Any unparsed command arguments will be passed on to pip\n'
        ),
    )
    subparsers = parser.add_subparsers(dest='command')

    install_parser = subparsers.add_parser(
        'install', help='Install packages',
        usage='derp',
    )
    add_shared_arguments(install_parser)

    wheel_parser = subparsers.add_parser('wheel', help='Build wheels')
    add_shared_arguments(wheel_parser)
    wheel_parser.add_argument(
        '-w', '--wheel-dir', help='Build wheels into this directory',
        default='./wheelhouse',
    )

    args, argv = parser.parse_known_args(argv)

    platform = args.platform.replace('-', '_')

    if args.command == 'install':
        return install(platform, argv)
    elif args.command == 'wheel':
        return wheel(platform, args.wheel_dir, argv)
    else:
        raise NotImplementedError('{0} not implemented'.format(args.command))


if __name__ == '__main__':
    exit(main())



"""Build packages to a wheel with a custom platform name"""
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import os
import shutil

from distlib.wheel import Wheel

from pip_custom_platform.util import mkdirp
from pip_custom_platform.util import tmpdir


def wheel(platform_name, wheel_dir, pip_args):
    mkdirp(wheel_dir)
    # Make the wheels using the platform default
    # Do this in a tempdir in case there are already wheels in the output
    # directory
    with tmpdir() as tempdir:
        import pip
        ret = pip.main(['wheel', '--wheel-dir', tempdir] + list(pip_args))
        if ret:
            return ret

        # Then rename any of the platform-specific wheels created
        for wheel_filename in os.listdir(tempdir):
            wheel_obj = Wheel(wheel_filename)

            if wheel_obj.arch != ['any']:
                wheel_obj.arch = [platform_name]
                dst = os.path.join(wheel_dir, wheel_obj.filename)
            else:
                dst = os.path.join(wheel_dir, wheel_filename)

            # And copy to the output directory
            shutil.copy(os.path.join(tempdir, wheel_filename), dst)



from __future__ import absolute_import
from __future__ import unicode_literals

import contextlib
import os
import shutil
import tempfile


@contextlib.contextmanager
def tmpdir():
    """Contextmanager to create a temporary directory.  It will be cleaned up
    afterwards.
    """
    tempdir = tempfile.mkdtemp()
    try:
        yield tempdir
    finally:
        shutil.rmtree(tempdir)


def mkdirp(path):
    try:
        os.makedirs(path)
    except OSError:
        if os.path.isdir(path):
            return
        else:
            raise



from setuptools import find_packages
from setuptools import setup

setup(
    name='pip_custom_platform',
    description=(
        'pip + wheel wrapper which allows you to choose a custom platform '
        'name for building, downloading, and installing wheels.'
    ),
    url='https://github.com/asottile/pip-custom-platform',
    version='0.1.0',
    author='Anthony Sottile',
    author_email='asottile@umich.edu',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
    packages=find_packages('.', exclude=('tests*', 'testing*')),
    install_requires=[
        'argparse',
        'distlib',
        'pip',
        'wheel',
    ],
    entry_points={
        'console_scripts': [
            'pip-custom-platform = pip_custom_platform.main:main',
        ],
    },
)



# pylint:disable=import-error,no-member,no-name-in-module
from __future__ import absolute_import
from __future__ import unicode_literals

import contextlib
import distutils.util


@contextlib.contextmanager
def patch_platform_name(platform_name):
    """The gross part: monkeypatching."""
    original_get_platform = distutils.util.get_platform
    distutils.util.get_platform = lambda: platform_name
    try:
        yield
    finally:
        distutils.util.get_platform = original_get_platform


def install(platform_name, pip_args):
    with patch_platform_name(platform_name):
        import pip
        return pip.main(['install'] + list(pip_args))
